/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
// Header file for CHist class

#ifndef _INC_CHIST_H
#define _INC_CHIST_H

#define CA_NAMESIZE 34

#include <stdarg.h>
#include <Xm/Xm.h>
#include "utils.h"
#include <tsDLList.h>
#include <CCaObject2.h>
#include <cvtFast.h>  // For cvtDoubleToString

typedef int (*CHistErrorHandler)(const char *fmt, ...);
typedef void (*CHistHelpCallback)(Widget w, XtPointer cd, XtPointer cbs);
typedef enum {
    SDDS_File,
    EPS_File
} FileType;

class COptions;

class CHist : public tsDLNode<CHist>
{
  public:
    CHist(char *pvNameIn);
    ~CHist(void);
    void initializeBins(void);
    void updatePlot();
    void updateXAxis(void);
    void setErrorHandler(CHistErrorHandler handler)
      {	errHandler=handler;}
    CHistErrorHandler getErrorHandler(void) const
      {	return errHandler; }
    CHistErrorHandler restoreErrorHandler(void)
      {	return (errHandler=errMsg); }
    void setHelpCB(CHistHelpCallback callback)
      {	helpCB=callback;}
    CHistHelpCallback getHelpCB(void) const
      {	return helpCB; }
    const char *getFileName(void) const { return fileName; }
    int writeSDDSFile(const char *fileNameIn, Boolean check=True);
    int writeEPSFile(const char *fileNameIn, Boolean check=True);

  private:
    void setBin(double val);
    void initialize(void);
    void startNewName(void);
    Widget createPlot(Widget parent);

    static int errMsg(const char *fmt, ...);
    static void fileSelectCB(Widget w, XtPointer clientData, XtPointer callData);
    static void executeFSD(CHist *hist, FileType fileTypeIn);
    static void start(Widget w, XtPointer clientData, XtPointer callData);
    static void stop(Widget w, XtPointer clientData, XtPointer callData);
    static void reset(Widget w, XtPointer clientData, XtPointer callData);
    static void updateOptionsDialog(CHist *hist, Boolean defaults);
    static void updateOptionsValues(CHist *hist, Boolean defaults);
    static void quit(Widget w, XtPointer clientData, XtPointer callData);
    static void mainCB(Widget w, XtPointer cd, XtPointer cbs);
    static void optionsBinCB(Widget w, XtPointer cd, XtPointer cbs);
    static void optionsAxisCB(Widget w, XtPointer cd, XtPointer cbs);
    static void helpCBWrapper(Widget w, XtPointer cd, XtPointer cbs);
    static CHistErrorHandler errHandler;
    static CHistHelpCallback helpCB;

  public:
    static void createOptionsDialog(CHist *hist, Boolean defaults);
    static void valueChangedCB(CCaObject2 *chanIn);
    static void graphicalInfoCB(CCaObject2 *chanIn);
    static void pollTimerProc(XtPointer cd, XtIntervalId *id);
    static void histTimerProc(XtPointer cd, XtIntervalId *id);

  private:
  // Options
    int binMode;
    double binUserMin;
    double binUserMax;
    unsigned binUpdatePeriod;  // ms
    int nBins;
    int axisMode;
    double axisXMin;
    double axisXMax;

    int nPoints;
    double binMin;
    double binMax;
    double binWidth;
    double binTotalWidth;
    double value;
    int initialized;

    CCaObject2 *chan;
    int needsUpdate;
    XtIntervalId histTimerId;
    double minY;
    double maxY;
    double *xArray;
    double *yArray;
    int yAxisStyle;
    int yRangeStyle;
    int plotID;
    Pixel bgColor;
    Pixel fgColor;
    char pvName[CA_NAMESIZE];
    double hopr;
    double lopr;
    short prec;
    int id;
    Widget shell;
    Widget mainForm;
    Widget plotW;
    char fileName[PATH_MAX];
    FileType fileType;
    
    static unsigned long refCount;
    static unsigned long nextId;
};

#endif // _INC_CHIST_H
